# -*- coding: utf-8 -*-

"""amda package for Space Physics WebServices Client."""

__author__ = """Alexis Jeandet"""
__email__ = 'alexis.jeandet@member.fsf.org'
__version__ = '0.1.0'

from .amda import AMDA, load_csv
